<?php /*

 ocPortal
 Copyright (c) ocProducts, 2004-2012

 See text/EN/licence.txt for full licencing information.


 NOTE TO PROGRAMMERS:
   Do not edit this file. If you need to make changes, save your changed file to the appropriate *_custom folder
   **** If you ignore this advice, then your website upgrades (e.g. for bug fixes) will likely kill your changes ****

*/

/**
 * @license		http://opensource.org/licenses/cpal_1.0 Common Public Attribution License
 * @copyright	ocProducts Ltd
 * @package		forum_blocks
 */

class Block_main_forum_news
{

	/**
	 * Standard modular info function.
	 *
	 * @return ?array	Map of module info (NULL: module is disabled).
	 */
	function info()
	{
		$info=array();
		$info['author']='Chris Graham';
		$info['organisation']='ocProducts';
		$info['hacked_by']=NULL;
		$info['hack_version']=NULL;
		$info['version']=2;
		$info['locked']=false;
		$info['parameters']=array('param','forum','member_based','date_key','title');
		return $info;
	}

	/**
	 * Standard modular cache function.
	 *
	 * @return ?array	Map of cache details (cache_on and ttl) (NULL: module is disabled).
	 */
	function cacheing_environment()
	{
		$info=array();
		$info['cache_on']='array(array_key_exists(\'title\',$map)?$map[\'title\']:\'\',array_key_exists(\'member_based\',$map)?$map[\'member_based\']:\'0\',array_key_exists(\'forum\',$map)?$map[\'forum\']:\'Announcements\',array_key_exists(\'param\',$map)?intval($map[\'param\']):14,array_key_exists(\'date_key\',$map)?$map[\'date_key\']:\'lasttime\')';
		$info['ttl']=60;
		return $info;
	}

	/**
	 * Standard modular run function.
	 *
	 * @param  array		A map of parameters.
	 * @return tempcode	The result of execution.
	 */
	function run($map)
	{
		if (has_no_forum()) return new ocp_tempcode();

		require_lang('news');
		require_css('news');
		require_code('xhtml');

		$num_topics=array_key_exists('param',$map)?intval($map['param']):14;
		$forum_name=array_key_exists('forum',$map)?$map['forum']:do_lang('NEWS');

		$num_topics=intval($num_topics);

		$date_key=array_key_exists('date_key',$map)?$map['date_key']:'lasttime';

		$rows=array();
		$archive_url=NULL;
		$submit_url=new ocp_tempcode();

		$forum_ids=array();
		if ((get_forum_type()=='ocf') && ((strpos($forum_name,',')!==false) || (preg_match('#\d[-\*\+]#',$forum_name)!=0) || (is_numeric($forum_name))))
		{
			require_code('ocfiltering');
			$forum_names=ocfilter_to_idlist_using_db($forum_name,'id','f_forums','f_forums','f_parent_forum','f_parent_forum','id',true,true,$GLOBALS['FORUM_DB']);
		} else
		{
			$forum_names=explode(',',$forum_name);
		}
		foreach ($forum_names as $forum_name)
		{
			$forum_name=is_integer($forum_name)?strval($forum_name):trim($forum_name);

			if ($forum_name=='<announce>')
			{
				$forum_id=NULL;
			} else
			{
				$forum_id=is_numeric($forum_name)?intval($forum_name):$GLOBALS['FORUM_DRIVER']->forum_id_from_name($forum_name);
			}

			if (!is_null($forum_id))
			{
				$forum_ids[$forum_id]=$forum_name;
				if (is_null($archive_url))
				{
					$archive_url=$GLOBALS['FORUM_DRIVER']->forum_url($forum_id,true); // First forum will count as archive
					if (get_forum_type()=='ocf')
					{
						$submit_url=build_url(array('page'=>'topics','type'=>'new_topic','id'=>$forum_id),get_module_zone('topics'));
					}
				}
			}
		}

//		if ((method_exists($GLOBALS['FORUM_DRIVER'],'supports_multi_show_forum_topics')) && ($GLOBALS['FORUM_DRIVER']->supports_multi_show_forum_topics()) && (count($forum_ids)!=0))
//		{
			$max_rows=0;
			$rows=$GLOBALS['FORUM_DRIVER']->show_forum_topics($forum_ids,$num_topics,0,$max_rows,'',true,$date_key);
			if (is_null($rows)) $rows=array();
/*		} else
		{
			foreach ($forum_ids as $forum_id=>$forum_name)
			{
				$max_rows=0;
				$t=$GLOBALS['FORUM_DRIVER']->show_forum_topics($forum_name,$num_topics,0,$max_rows,'',true);
				if (!is_null($t)) $rows=array_merge($rows,$t);
			}
		}*/

		global $M_SORT_KEY;
		$M_SORT_KEY=$date_key;
		usort($rows,'multi_sort');
		$rows=array_reverse($rows,false);

		$_title=do_lang_tempcode('NEWS');
		if ((array_key_exists('title',$map)) && ($map['title']!='')) $_title=protect_from_escaping(escape_html($map['title']));

		$i=0;
		$news_text=new ocp_tempcode();
		while (array_key_exists($i,$rows))
		{
			$myrow=$rows[$i];

			$id=$myrow['id'];
			$date=get_timezoned_date($myrow[$date_key]);
			$author_url=(((array_key_exists('member_based',$map)) && ($map['member_based']=='1')) || (!addon_installed('authors')))?new ocp_tempcode():build_url(array('page'=>'authors','type'=>'misc','author'=>$myrow['firstusername']),get_module_zone('authors'));
			$author=$myrow['firstusername'];
			$news_title=escape_html($myrow['title']);
			$news=is_object($myrow['firstpost'])?$myrow['firstpost']:make_string_tempcode(xhtmlise_html($myrow['firstpost']));
			if (is_null($news)) $news='';
			$full_url=$GLOBALS['FORUM_DRIVER']->topic_url($id,'',true);
			$news_text->attach(do_template('NEWS_BOX',array('_GUID'=>'2edf18daf5510495fd588cad062aec4e','TRUNCATE'=>false,'BLOG'=>false,'FIRSTTIME'=>strval($myrow['firsttime']),'LASTTIME'=>strval($myrow['lasttime']),'CLOSED'=>strval($myrow['closed']),'FIRSTUSERNAME'=>$myrow['firstusername'],'LASTUSERNAME'=>$myrow['lastusername'],'FIRSTMEMBERID'=>strval($myrow['firstmemberid']),'LASTMEMBERID'=>strval($myrow['lastmemberid']),'ID'=>strval($id),'FULL_URL'=>$full_url,'SUBMITTER'=>strval($myrow['firstmemberid']),'DATE'=>$date,'DATE_RAW'=>strval($myrow[$date_key]),'NEWS_TITLE'=>$news_title,'CATEGORY'=>'','IMG'=>'','AUTHOR'=>$author,'AUTHOR_URL'=>$author_url,'NEWS'=>$news)));

			$i++;

			if ($i==$num_topics) break;
		}
		if ($news_text->is_empty())
		{
			return do_template('BLOCK_NO_ENTRIES',array('_GUID'=>'f55c90205b4c80162494fc5e2b565ce6','HIGH'=>false,'TITLE'=>$_title,'MESSAGE'=>do_lang_tempcode('NO_NEWS'),'ADD_NAME'=>do_lang_tempcode('ADD_TOPIC'),'SUBMIT_URL'=>$submit_url));
		}

		if (is_null($forum_id)) $archive_url='';

		return do_template('BLOCK_MAIN_FORUM_NEWS',array('_GUID'=>'36b05da9aed5a2056bdb266e2ce4be9f','TITLE'=>$_title,'FORUM_NAME'=>array_key_exists('forum',$map)?$map['forum']:do_lang('NEWS'),'CONTENT'=>$news_text,'BRIEF'=>new ocp_tempcode(),'ARCHIVE_URL'=>$archive_url,'SUBMIT_URL'=>$submit_url,'RSS_URL'=>'','ATOM_URL'=>''));
	}

}


